(ns clj-ml2.ols
  (:use clojure.core.matrix
        clj-ml2.multivar
        [incanter.core :only [bind-columns]])
  (:require [clojure.core.matrix.operators :as M]
            [clatrix.core :as cl]))

(defn linear-model-ols
  "Estimates the coefficients of a multi-var linear
  regression model using Ordinary Least Squares (OLS) method"
  [MX MY]
  (let [X (bind-columns (repeat (row-count MX) 1) MX)
        Xt (transpose X)
        Xt-X (cl/* Xt X)]
    (cl/* (inverse Xt-X) Xt MY)))

;; (defn linear-model-ols
;;   "Estimates the coefficients of a multi-var linear regression model
;;    using Oridinary Least Squares (OLS) method"
;;   [MX MY]
;;   (let [X (bind-columns (repeat (row-count MX) 1) MX)
;;         Xt (transpose X)
;;         Xt-X (M/* Xt X)]
;;     (M/* (inverse Xt-X) Xt MY)))

(def ols-linear-model
  (linear-model-ols X Y))

(def ols-linear-model-coefs
  (cl/as-vec ols-linear-model))

;; user> (cl/as-vec (ols-linear-model X Y))
;; [1.851198344985435 0.6252788163253274 0.7429244752213087 -0.4044785456588674 -0.22635635488532463]
;; user> (:coefs iris-linear-model)
;; [1.851198344985515 0.6252788163253129 0.7429244752213329 -0.40447854565877606 -0.22635635488543926]
;; user> (every? #(< % 0.0001)
;;               (map - ols-linear-model-coefs (:coefs iris-linear-model))
;; true

(defn predict [coefs X]
  {:pre [(= (count coefs)
            (+ 1 (count X)))]}
  (let [X-with-1 (conj X 1)
        products (map * coefs X-with-1)]
    (reduce + products)))
