(ns clj-ml2.scatter
  (:gen-class)
  (:use [clojure.core.matrix :only [transpose]]
        [incanter.charts :only [scatter-plot xy-plot add-lines]]
        [incanter.stats :only [linear-model quantile-normal]]
        [incanter.core :only [view sel sqrt matrix]])
  (:require [clatrix.core :as cl]))

;; generated by
;; (def mvn-samp (sample-mvn 10 :mean [11 0] :sigma (matrix [[11 2] [2 1]])))

;; (def linear-samp
;;   (cl/matrix [[ 8.401 -1.570]
;;               [14.475  2.320]
;;               [13.396  0.424]
;;               [12.127  0.814]
;;               [ 5.044 -2.300]
;;               [ 8.339  0.009]
;;               [15.692  1.954]
;;               [17.108  2.296]
;;               [ 9.253 -0.635]
;;               [12.029  0.328]]))

;; (def X (sel linear-samp :cols 0))
;; (def Y (sel linear-samp :cols 1))

(def X (cl/matrix [8.401 14.475 13.396 12.127 5.044
                   8.339 15.692 17.108 9.253 12.029]))

(def Y (cl/matrix [-1.57 2.32  0.424  0.814 -2.3
                    0.01 1.954 2.296 -0.635 0.328]))

;; create scatter-plot of points
(def linear-samp-scatter
  (scatter-plot X Y))

(defn plot-scatter []
  (view linear-samp-scatter))

(def samp-linear-model
  (linear-model Y X))

;; to get the coefficients of the model,
;; (:coefs samp-linear-model)

;; to get residuals of the model,
;; user> (:residuals samp-linear-model)
;; [-0.6873445559690581 0.8253111334125092 -0.6483716931997257 0.2383108767994172 -0.10342541689331242 0.9169220471357067 -0.01701880172457293 -0.22923670489146497 -0.08581465024744239 -0.20933223442208365]

;; to get SSE,
;; user> (:sse samp-linear-model)
;; 2.5862250345284887

;; to get coefficient of determination,
;; user> (:r-square samp-linear-model)
;; 0.8837893226172282

(defn plot-model []
  (view (add-lines linear-samp-scatter X (:fitted samp-linear-model))))

(def norm-scatter-samp
  (let [samp (mapv #(rand %1) (repeat 10 1))]
    (transpose
     (cl/matrix
      [samp (quantile-normal samp :mean -5 :sd (sqrt 2))]))))

(def norm-X (sel norm-scatter-samp :cols 0))
(def norm-Y (sel norm-scatter-samp :cols 1))

(defn plot-norm-model []
  (view (add-lines (scatter-plot norm-X norm-Y) norm-X
                   (:fitted (linear-model norm-Y norm-X)))))

(defn -main [& args]
  (println "Hello"))

;; parabolic cost function

(defn pbola [x]
  (+ (- (* x x) (* x 6.6)) 16))

(defn plot-pbola []
  (let [x (range -10 20 0.1)
        y (map pbola x)]
    (view (xy-plot x y
                   :x-label "Parameter β of the model"
                   :y-label "Cost function J(β)"))))


;; gradient descent implementation

(def gradient-descent-precision 0.001)

(defn gradient-descent
  "Find the local minimum of the cost function's plot"
  [F' x-start step]
  (loop [x-old x-start]
    (let [x-new (- x-old
                   (* step (F' x-old)))
          dx (- x-new x-old)]
      (if (< dx gradient-descent-precision)
        x-new
        (recur x-new)))))

;; f(x) = x^4 − 3x^3 + 2, with derivative f'(x) = 4x^3 − 9x^2
(defn f' [x]
  (- (* 4 x x x)
     (* 9 x x)))
;; #'user.scatter/f'

;; user> (gradient-descent f' 6 0.01)
;; 0.5999999999999996
