(ns clj-ml3.decision-tree
  (:use clj-ml3.data
        [clj-ml classifiers data]))

;; Decision tree classifier using clj-ml

(def DT-classifier (make-classifier :decision-tree :c45))

(defn train-DT-classifier []
  (dataset-set-class fish-dataset 0)
  (classifier-train DT-classifier fish-dataset))

;; user> (train-DT-classifier)
;; #<J48 J48 pruned tree
;; ------------------

;; width <= 3: sea-bass (2320.0)
;; width > 3
;; |   length <= 6
;; |   |   lightness <= 5: salmon (7147.0/51.0)
;; |   |   lightness > 5: sea-bass (95.0)
;; |   length > 6: sea-bass (438.0)

;; Number of Leaves  : 	4

;; Size of the tree : 	7
;; >

;;

;; user> (classifier-classify DT-classifier sample-fish)
;; :salmon

(def UDT-classifier (make-classifier :decision-tree :c45 {:unpruned true}))

(defn train-UDT-classifier []
  (dataset-set-class fish-dataset 0)
  (classifier-train UDT-classifier fish-dataset))

;; user> (train-UDT-classifier)
;; #<J48 J48 unpruned tree
;; ------------------

;; width <= 3: sea-bass (2320.0)
;; width > 3
;; |   length <= 6
;; |   |   lightness <= 5
;; |   |   |   length <= 5: salmon (6073.0)
;; |   |   |   length > 5
;; |   |   |   |   width <= 4
;; |   |   |   |   |   lightness <= 3: salmon (121.0)
;; |   |   |   |   |   lightness > 3
;; |   |   |   |   |   |   lightness <= 4: salmon (52.0/25.0)
;; |   |   |   |   |   |   lightness > 4: sea-bass (50.0/24.0)
;; |   |   |   |   width > 4: salmon (851.0)
;; |   |   lightness > 5: sea-bass (95.0)
;; |   length > 6: sea-bass (438.0)

;; Number of Leaves  : 	8

;; Size of the tree : 	15
;; >

;;

;; user> (classifier-classify UDT-classifier sample-fish)
;; :salmon
