(ns clj-ml5.cv
  (:use [clj-ml classifiers data]))

;; Data for fish packaging plant example

(defn rand-in-range
  "Generates a random integer within the given range"
  [min max]
  (let [len (- max min)
        rand-len (rand-int len)]
    (+ min rand-len)))

;; sea bass are mostly long and light in color
(defn make-sea-bass []
  (vector :sea-bass
          (rand-in-range 6 10)          ; length
          (rand-in-range 0 5)           ; width
          (rand-in-range 4 10)))        ; lightness of skin

;; salmon are mostly fat and dark
(defn make-salmon []
  (vector :salmon
          (rand-in-range 0 7)           ; length
          (rand-in-range 4 10)          ; width
          (rand-in-range 0 6)))         ; lightness of skin

(defn make-sample-fish []
  (if (< (rand) 0.3) (make-sea-bass) (make-salmon)))

(def fish-training-data
  (for [i (range 10000)] (make-sample-fish)))

(def fish-template
  [{:category [:salmon :sea-bass]}
   :length :width :lightness])

(def fish-dataset
  (make-dataset "fish" fish-template fish-training-data))

(def *classifier* (make-classifier :decision-tree :c45))

(defn train-classifier []
  (dataset-set-class fish-dataset 0)
  (classifier-train *classifier* fish-dataset))

(def fish-cv-data
  (for [i (range 3000)] (make-sample-fish)))

(def fish-cv-dataset
  (make-dataset "fish-cv" fish-template fish-cv-data))

(defn cv-classifier [folds]
  (dataset-set-class fish-cv-dataset 0)
  (classifier-evaluate *classifier* :cross-validation
                       fish-cv-dataset folds))

;; user> (def cv (cv-classifier 10))
;; #'user/cv

;;;

;; user> (print (:summary cv))

;; Correctly Classified Instances        2986               99.5333 %
;; Incorrectly Classified Instances        14                0.4667 %
;; Kappa statistic                          0.9888
;; Mean absolute error                      0.0093
;; Root mean squared error                  0.0681
;; Relative absolute error                  2.2248 %
;; Root relative squared error             14.9238 %
;; Total Number of Instances             3000
;; nil

;;;

;; user> (print (:confusion-matrix cv))
;; === Confusion Matrix ===

;;     a    b   <-- classified as
;;  2129    0 |    a = salmon
;;     9  862 |    b = sea-bass
;; nil
