(ns clj-ml6.linear-svm
  (:use [clj-liblinear.core :only [train predict]]
        [clojure.string :only [split lower-case]]))

(def training-data
  (concat
   (repeatedly
    500 #(hash-map :class 0
                   :data {:x (rand)
                          :y (rand)}))
   (repeatedly
    500 #(hash-map :class 1
                   :data {:x (- (rand))
                          :y (- (rand))}))))

;; user> (first training-data)
;; {:class 0,
;; :data {:x 0.054125811753944264, :y 0.23575052637986382}}
;; user> (last training-data)
;; {:class 1,
;; :data {:x -0.8067872409710037, :y -0.6395480020409928}}

(defn train-svm []
  (train
   (map :data training-data)
   (map :class training-data)))

;; user> (def svm (train-svm))
;; #'user/svm
;; user> (predict svm {:x 0.5 :y 0.5})
;; 0.0
;; user> (predict svm {:x -0.5 :y 0.5})
;; 0.0
;; user> (predict svm {:x -0.4 :y 0.4})
;; 0.0
;; user> (predict svm {:x -0.4 :y -0.4})
;; 1.0
;; user> (predict svm {:x 0.5 :y -0.5})
;; 1.0

(def tweets
  [{:class 0 :text "new lisp project released"}
   {:class 0 :text "try out this emacs package for common lisp"}
   {:class 0 :text "a tutorial on guile scheme"}

   {:class 1 :text "update in javascript library"}
   {:class 1 :text "node.js packages are now supported"}
   {:class 1 :text "check out this jquery plugin"}

   {:class 2 :text "linux kernel news"}
   {:class 2 :text "unix man pages"}
   {:class 2 :text "more about linux software"}])

(defn extract-words [text]
  (->> #" "
       (split text)
       (map lower-case)
       (into #{})))

;; user> (extract-words "some text to extract some words")
;; #{"extract" "words" "text" "some" "to"}

(defn train-svm []
  (train (->> tweets
              (map :text)
              (map extract-words))
         (map :class tweets)))

(defn predict-svm [svm text]
  (predict
    svm (extract-words text)))


;; user> (def svm (train-svm))
;; #'user/svm
;; user> (predict-svm svm "a common lisp tutorial")
;; 0.0
;; user> (predict-svm svm "new javascript library")
;; 1.0
;; user> (predict-svm svm "new linux kernel update")
;; 2.0
