(ns clj-ml7.som-cluster
  (:use [incanter core som stats charts datasets]))

(def iris-features (to-matrix (sel (get-dataset :iris)
                                   :cols [:Sepal.Length
                                          :Sepal.Width
                                          :Petal.Length
                                          :Petal.Width])))

(def som (som-batch-train
          iris-features :cycles 10 :alpha 0.5 :beta 3))

;; user> (:dims som)
;; [10.0 2.0]

;; user> (:sets som)
;; {[4 1] (144 143 141 ... 102 100),
;;  [8 1] (149 148 147 ... 50),
;;  [9 0] (49 48 47 46 ... 0)}

;; plot the means of the feature vectors in each cell/cluster
(def feature-mean
  (map #(map mean (trans
                   (sel iris-features :rows ((:sets som) %))))
       (keys (:sets som))))

(defn plot-means []
  (let [x (range (ncol iris-features))
        cluster-name #(str "Cluster " %)]
    (-> (xy-plot x (nth feature-mean 0)
                 :x-label "Feature"
                 :y-label "Mean value of feature"
                 :legend true
                 :series-label (cluster-name 0))
        (add-lines x (nth feature-mean 1)
                   :series-label (cluster-name 1))
        (add-lines x (nth feature-mean 2)
                   :series-label (cluster-name 2))
        view)))

;; user> (plot-means)

(defn print-clusters []
  (doseq [[pos rws] (:sets som)]
    (println pos \:
             (frequencies
              (sel (get-dataset :iris)
                   :cols :Species :rows rws)))))

;; user> (print-clusters)
;; [4 1] : {virginica 23}
;; [8 1] : {virginica 27, versicolor 50}
;; [9 0] : {setosa 50}
;; nil
