(ns clj-ml9.bigml
  (:require [bigml.api [core :as api]
             [source :as source]
             [dataset :as dataset]
             [model :as model]
             [prediction :as prediction]
             [evaluation :as evaluation]]))


;; !!! WARNING: sends data to cloud !!!

;;; sign up for a free BigML account here
;;; https://bigml.com/accounts/register/

(def default-connection
  (api/make-connection
   "darth10"                                   ; username
   "5301595fa2ee1960428a69335a4ac66664b8b34b"  ; API key
   true))                                      ; dev/production mode

(defmacro with-default-connection [& body]
  `(api/with-connection default-connection
     ~@body))

(def default-source
  (with-default-connection
    (source/create [["Make"  "Model"   "Year" "Weight" "MPG"]
                    ["AMC"   "Gremlin" 1970   2648     21]
                    ["AMC"   "Matador" 1973   3672     14]
                    ["AMC"   "Gremlin" 1975   2914     20]
                    ["Honda" "Civic"   1974   2489     24]
                    ["Honda" "Civic"   1976   1795     33]])))

;;;

(def default-dataset
  (with-default-connection
    (api/get-final (dataset/create default-source))))

(def default-model
  (with-default-connection
    (api/get-final (model/create default-dataset))))

;;;

(def default-remote-prediction
  (with-default-connection
    (prediction/create default-model [1973 3672])))

;; user> (:prediction default-remote-prediction)
;; {:000004 33}

;;;

(def default-local-predictor
  (with-default-connection
    (prediction/predictor default-model)))

;; user> (default-local-predictor [1983])
;; 22.4
;; user> (default-local-predictor [1983] :details true)
;; {:prediction {:000004 22.4},
;;  :confidence 24.37119, :count 5, :id 0,
;;  :objective_summary {:counts [[14 1] [20 1] [21 1] [24 1] [33 1]]}}

(defn final-with-default-connection [f & xs]
  (with-default-connection
    (api/get-final (apply f xs))))

;;;

(defmacro get-final-> [head & body]
  (let [final-body (map list
                        (repeat 'final-with-default-connection)
                        body)]
    `(->> ~head
          ~@final-body)))

;;;

(def iris-model
  (get-final-> "https://static.bigml.com/csv/iris.csv"
               source/create
               dataset/create
               model/create))

(def iris-evaluation
  (with-default-connection
    (api/get-final
     (evaluation/create iris-model (:dataset iris-model)))))

;; user> (-> iris-evaluation :result :model :accuracy)
;; 1
