(ns clj-ml9.datastore
  (:use incanter.core
        [incanter.mongodb :only [insert-dataset fetch-dataset]]
        [somnium.congomongo :only [mongo!]]
        [incanter.datasets :only [get-dataset]]))

(def iris (get-dataset :iris))

;; user> (with-data iris
;;         (view (conj-cols (range (nrow $data)) $data)))

;;;

;; user> (with-data iris ($ [:Species :Sepal.Length]))


;; |   :Species | :Sepal.Length |
;; |------------+---------------|
;; |     setosa |           5.1 |
;; |     setosa |           4.9 |
;; |     setosa |           4.7 |
;;   ...
;; |  virginica |           6.5 |
;; |  virginica |           6.2 |
;; |  virginica |           5.9 |

;; user> (with-data iris ($ [:Species :Sepal.Length]
;;                          ($where {:Sepal.Length 7.7})))

;; |  :Species | :Sepal.Length |
;; |-----------+---------------|
;; | virginica |           7.7 |
;; | virginica |           7.7 |
;; | virginica |           7.7 |
;; | virginica |           7.7 |

;;;

;; user> (with-data iris ($ [:Species :Sepal.Length]
;;                          ($where {:Sepal.Length {:$gt 7.0}})))

;; |  :Species | :Sepal.Length |
;; |-----------+---------------|
;; | virginica |           7.1 |
;; | virginica |           7.6 |
;; | virginica |           7.3 |
;;   ...
;; | virginica |           7.2 |
;; | virginica |           7.2 |
;; | virginica |           7.4 |

;;;

;; user> (with-data iris ($ [:Species :Sepal.Length]
;;                          ($where {:Sepal.Length
;;                                   {:$gt 7.0 :$lt 7.5}})))

;; |  :Species  |:Sepal.Length |
;; |------------+--------------|
;; | virginica  |          7.1 |
;; | virginica  |          7.3 |
;; | virginica  |          7.2 |
;; | virginica  |          7.2 |
;; | virginica  |          7.2 |
;; | virginica  |          7.4 |

;; ($where {:speed {:$in #{4 7 24 25}}} results)
;; ($where {:speed {:$nin #{4 7 24 25}}} results)

;; user> (mongo! :db "sampledb")
;; true
;;; can use :host and :port key args

;; user> (insert-dataset
;;        :iris (col-names iris [:SepalLength
;;                               :SepalWidth
;;                               :PetalLength
;;                               :PetalWidth
;;                               :Species]))

;; user> (with-data (fetch-dataset :iris) ($ [:Species :_id]
;;                                           ($where {:SepalLength
;;                                                    {:$gt 7}})))

;; |  :Species |                     :_id |
;; |-----------+--------------------------|
;; | virginica | 52ebcc1144ae6d6725965984 |
;; | virginica | 52ebcc1144ae6d6725965987 |
;; | virginica | 52ebcc1144ae6d6725965989 |
;;   ...
;; | virginica | 52ebcc1144ae6d67259659a0 |
;; | virginica | 52ebcc1144ae6d67259659a1 |
;; | virginica | 52ebcc1144ae6d67259659a5 |


;;;
;;; MongoDB shell
;;;

;;; $ mongo
;;; MongoDB shell version: 2.4.6
;;; connecting to: test
;;; Server has startup warnings:
;;; ...
;;;
;;; > use sampledb
;;; switched to db sampledb


;;;

;;; > show collections
;;; iris
;;; system.indexes
;;; >
;;; > db.iris.find()
;;; > db.iris.find( { SepalLength: 5})
;;;
;;; { "_id" : ObjectId("52ebcc1144ae6d6725965922"),
;;;   "Species" : "setosa",
;;;   "PetalWidth" : 0.2,
;;;   "PetalLength" : 1.4,
;;;   "SepalWidth" : 3.6,
;;;   "SepalLength" : 5 }
;;; { "_id" : ObjectId("52ebcc1144ae6d6725965925"),
;;;   "Species" : "setosa",
;;;   "PetalWidth" : 0.2,
;;;   "PetalLength" : 1.5,
;;;   "SepalWidth" : 3.4,
;;;   "SepalLength" : 5 }
;;;
;;; ...
